
## Introduction to DMS3

DMS3 stands for **Dis-intermediated Multi-State Space Search**.

DMS3 is a p2p network protocol that enables ***Search*** for
information published by peer nodes.

Peer nodes use DMS3 to index information into local repositories that enable context specific search for various kinds of information. For each indexed information repository, peer nodes also index metadata repositories used to provide context on locally hosted information. Metadata repositories also act as source testament to the nature of information hosted.

Metadata repositories enable a search client to self-sensor inappropriate
information while allowing the publisher freedom of speech. A metadata
repository acts as a manifest for hosted information repositories.

Metadata repositories enable network users to flag inconsistencies between
published information and its manifest as an aid to expose deceptive peer
nodes practices. Deceptive peer nodes can be quarantined and penalized by
network protocols subject to arbitration resolution protocols.

***Multi-State Space Search*** refers to the ability to separately search
context specific manifest and information repositories.

Information flow is ***Dis-intermediated***. Information is neither aggregated
by a central service, nor filtered or rank ordered by a central service.

### Key tenets

#### Decentralized control

DMS3 is a permission-less peer-to-peer (p2p) network of nodes that join and
leave the network at will without a central authority enforcing admission control.

#### Dis-intermediated service

DMS3 services are transacted directly between participant nodes without third
party mediation.

#### Transparency of application and information

Published DMS3 integrated applications are audited for transparency and security, such applications use the DMS3 API to store and publish generated persistent information.

Applications run on endpoint nodes and store all local activity data on the
local endpoint node via the DMS3 API.

There are no web servers to collect any personal activity data using the application.

All data sharing is controlled by the authorized endpoint node user. The
network may offer optional centralized services for data availability, reliability, safety, and recovery. Additional future, new specialized network services may also be provided where implementing such services at each endpoint node is not capital efficient.

#### Sovereignty and privacy of personal information

The endpoint node user has full control over information sharing. Publishing information is explicitly controlled at each endpoint node by opt-in mechanisms. All information published by the endpoint node user is subject to transparency.

Endpoint identity (ex: wallet account, node id, etc.) is transparent and opaque using public key cryptography mechanisms. Responsibility for  securing published information and personal endpoint accounts rests with  the endpoint user.

#### Security in an untrusted environment

Endpoint nodes in the p2p network are not trusted, trust is provided by the network layer using blockchain technology.

#### Information availability and reliability

The network implements optional centralized, permission-ed, shared nothing distributed architecture to provide information distribution and recovery services to protect against loss or availability of information.

The p2p network uses *Smart Contracts* to enforce distribution and recovery
service agreements between an endpoint and network services.

#### Censorship, freedom, and acceptable speech

Decentralization and Dis-intermediation enables freedom of speech.

The network enables consumers to self-sensor inappropriate speech while
enabling publishers the freedom of speech.

Acceptable speech is defined by each endpoint and in accordance to locally
interpreted cultural norms.

The network protocols provide mechanisms to quarantine and penalize peer nodes that violate the acceptable use policy.

#### Balancing individual and group rights

In an ideal governance environment, information flow and sharing should be maximized and be free for the benefit of the group.

Innovation accumulates knowledge, sharing drives and accelerates wealth and economic growth.

Individuals share more of their knowledge and wealth when they perceive their basic needs are secure and growth objectives achievable. Individual needs include:

1. Food
2. Shelter
3. Health
4. Security, safety, privacy
5. Fair share
6. Continuity
7. Acceptance

Due to inefficiencies in our current governance practices and lack of guarantees on individual needs, information flow is sub-optimal and free sharing often (not always) limited to self promotion and gain. Where self may be an individual or a segment of society or group.

DMS3 will be open source software with certain proprietary components and services developed and non-free to meet individual needs.
